\name{pipe}

\alias{pipe}
\alias{initPipe}

\title{Set the basic properties of a pipe}

\description{The function \code{pipe} creates an object of class
\code{pipe} consisting of the basic physical properties of the
pipe, while \code{initPipe} sets the object to its steady state
values.}

\usage{
pipe(L, D, A, G=9.8)
initPipe(pobj, fc, H, K, even = TRUE) 
}

\arguments{
  \item{L}{Length of pipe}
  \item{D}{Diameter of pipe}
  \item{A}{Celerity of liquid in pipe}
  \item{G}{Graviational constant}
  \item{pobj}{Object of class \code{pipe}}
  \item{fc}{Vector of friction coefficients}
  \item{H}{Upstream piezometric head}
  \item{K}{Downstream valve loss coefficient}
  \item{even}{Is the current time-step even or odd?}
}

\value{The function \code{pipe} creates an object of class
\code{pipe} which is simply a list of the parameters.  The
function \code{initPipe} adds information on the steady state
pressure and flow (components \code{pres} and \code{flow}) for
the choice of \code{fc}, \code{H} and \code{K}.}

\details{

\bold{Friction coefficients.}  The pipe is divided into \code{n}
reaches, where \code{n} must be odd (for technical reasons).  The
vector \code{fc} contains \code{n} friction coefficients each
defined as \deqn{fc_i = \int_0^{\Delta x} \mu(x_i+y, t+y)
dy}{fc_i = int_0^{Delta x} mu(x_i+y, t+y) dy} where \eqn{x_i}
denotes the location of the start of reach \eqn{i}.  The function
\eqn{\mu(x, t)}{mu(x, t)} denotes the Darcy-Weisbach friction
factor at location \eqn{x} and time \eqn{t}.  In the trivial case
where \eqn{\mu(x, t) = \bar{\mu}}{mu(x, t) = bar{mu}}, a
constant, then \eqn{fc_i = \bar{\mu} \Delta x}{fc_i = bar{mu}
Delta x}, where \eqn{\Delta x = L / n}{Delta x = L / n}.

\bold{Wobbly grid.} For technical reasons the flow equations are
solved on a diagonal grid.  This is arranged to intersect the
upstream boundary at even time-points, and the downstream
boundary at odd timepoints.  If \code{x} denotes the \code{n+1}
vertices in the space grid, then at any time point the values at
locations \code{x[seq(from=2-even, by=2, len=(n+1)/2)]} will be
represented in the \code{head} and \code{flow} components, where
\code{even} is logical.  Hence the necessity to decide (usually
arbitrarily) whether the initial time point is even or odd.
Regular grids will require only alternate time-points to be used,
while a regular grid that spans the pipe length will require some
form of interpolation.

\bold{Boundary conditions.}  The upstream boundary condition
specifies the piezometric head at the upstream end of the pipe
(relative to the downstream end) modelling, for example, a large
reservoir of known (and possibly varying) depth.  The downstream
boundary consists of a valve modelled by a valve-loss coefficient
\code{K}, which describes the head loss at the valve as \deqn{h =
K v^2 / (2 * G)} where \code{v} is the velocity through the
valve.  Large positive values of \code{K} denote a partially
closed valve, while an infinite, \code{NA} or negative value
denotes a completely closed valve.  If \code{K=0} then the valve
is wide-open.
}

\references{
Massey, B.S, Mechanics of Fluids, 1989, Chapman and Hall,
particularly chapters 7 and 13.

Wylie, E.B., and Streeter, V.L., 1993, Fluid Transients in
Systems, Prenctice-Hall, chapters 1-3.

Rougier, J.C., and Goldstein, M., 2001, A Bayesian Analysis of
Fluid Flow in Pipe-lines, Journal of the Royal Statistical
Society, Series C.
}

\author{Jonathan Rougier, \email{J.C.Rougier@durham.ac.uk}}

\seealso{\code{\link{stepPipe}}, for advancing the state of the
pipe by one period.}

\examples{
# from Massey, Example 2 in ch. 7,
pp <- pipe(L=120, D=0.1, A=1231) # A for kerosene
n <- 21
dx <- pp$L / n
fc <- rep(4*0.0079*dx, n)
pp <- initPipe(pp, fc=fc, H=5, K=0)
}

\keyword{misc}
