\name{stepPipe}

\alias{stepPipe}

\title{Advance the state of the pipe}

\description{A wrapper for the C function "onestep" that advances
the pressure and flow data in an initialised object of class
\code{pipe} according to the given boundary conditions \code{H}
or \code{K} (according to whether the notional time-point is even
or odd: see the help file for \code{initPipe}), and depending
upon whether there is a leak (the default is no leak).}

\usage{
stepPipe(pobj, fc, H, K, xleak=NA, pleak=0)
}

\arguments{
  \item{pobj}{Object of class \code{pipe}}
  \item{fc}{Vector of friction coefficients}
  \item{H}{Upstream piezometric head}
  \item{K}{Downstream valve loss coefficient}
  \item{xleak}{Location of leak (if applicable)}
  \item{pleak}{Proportionate loss at leak (could be zero)}
}

\value{An updated object of class \code{pipe}.}

\author{Jonathan Rougier, \email{J.C.Rougier@durham.ac.uk}}

\seealso{\code{\link{initPipe}}, for further details.}

\examples{
pp <- pipe(L=1e4, D=0.1, A=1231)
n <- 51
dx <- pp$L / n
fc <- rep(4*0.0079*dx, n)

# random walk at upstream boundary

pp <- initPipe(pp, fc=fc, H=50, K=100)
usb <- pp$head[1] * exp(cumsum(rnorm(200, 0.0, 0.01)))
pic <- numeric(0)
for (H in usb) {
  pp <- stepPipe(pp, fc=fc, H=H, K=100)
  if (pp$even) pic <- cbind(pic, pp$flow)
}

image(
  x = seq(from=0, by=2*pp$L/pp$n, len=(n+1)/2),
  y = 1:ncol(pic)*2*pp$dt,
  z = pic,
  xlab = "Distance along the pipe (m)",
  ylab = "Time (s)",
  main = "Flow (m/s), random upstream boundary"
)
lines(
  x = ((usb-min(usb))/diff(range(usb)) * pp$L/10),
  y = seq(along=usb)*pp$dt
)
box()

# sudden development of leak

pp <- initPipe(pp, fc=fc, H=50, K=100)
xleak <- 0.4 * pp$L
lsz <- c(seq(from=0, to=0.15, len=50), rep(0.15, 150))
pic <- numeric(0)
for (pleak in lsz) {
  pp <- stepPipe(pp, fc=fc, H=50, K=100, xleak=xleak, pleak=pleak)
  if (pp$even) pic <- cbind(pic, pp$flow)
}

image(
  x = seq(from=0, by=2*pp$L/pp$n, len=(n+1)/2),
  y = 1:ncol(pic)*2*pp$dt,
  z = pic,
  xlab = "Distance along the pipe (m)",
  ylab = "Time (s)",
  main = "Flow (m/s), Rapid development of a leak"
)
box()
}

\keyword{misc}

