/* One step advance using waterhammer equations and wobbly grid

   Input variables are all pointers to ...

   thIn  Input state vector, comprising (n+1)/2 heads and (n+1)/2
         flows concatenated
   thOut Output state vector, as above
   fc    n friction coefficient values
   n     number of reaches (integer, odd)
   even  boolean (t even is downstream boundary)
   L     pipe length
   D     pipe diameter
   A     pipe celerity
   G     gravitational constant
   H     Upstream head for next time point (only matters for even==0)
   K     Downstream valve coefficient (only matters for even==1)
   xleak Location of leak (if one exists)
   pleak Proportionate loss through leak (zero if no leak)

   Algorithm based on Wylie and Streeter, 1993, Fluid Transients
   in Systems, Prentice Hall, Inc., pp 39-42.

   See also Rougier and Goldstein, 2001, A Bayesian Analysis of
   Fluid Flow in Pipelines, Journal of the Royal Statistical
   Society, Series C.  The friction coefficient "fc" starting at
   location x_i is the line integral \int_0^{Dx} \mu(x_i+y, t+y)
   dy where \mu(x, t) is the Darcy-Weisbach friction factor at
   location x and time t.

   The leak is modelled by destroying mass (ie velocity) on those
   characteristics that pass through the leak location.

   */

#include <R.h>

void onestep(double *thOut, const double *thIn, const double *fc,
	     const int *n, const int *even, const double *L,
	     const double *D, const double *A, const double *G, 
	     const double *H, const double *K, const double *xleak,
	     const double *pleak)
{
  double CP, ABP, CM, ABM, cc1, cc2, pp, pm;
  int i, j, fcj, v, isEven, isLeak, ileak, isPos;

  /* set up constants */

  if (*n % 2 == 0)
    error("Expecting odd value for \"n\"");
  v = (int) (*n + 1) / 2;

  cc1 = 1.0 / (2.0 * *G * *D);
  cc2 = *A / *G;
  isEven = *even != 0;
  isLeak = *pleak > 0.0;

  if (isLeak) {
    ileak = (int) floor(*xleak / (*L / *n));
    isPos = isEven ? ileak % 2 == 0 : ileak % 2 == 1;
    ileak = (ileak + (isEven ? 0 : 1)) / 2;
  }

  i = 0;

  /* handle upstream boundary if odd time-point */

  if (!isEven) {

    if (isLeak && ileak==0)
      pm = *pleak;
    else pm = 0.0;

    thOut[0] = *H;
    CM = thIn[0] - cc2 * thIn[v] / (1.0-pm);
    ABM = cc2 + fc[0] * cc1 * fabs(thIn[v] / (1.0-pm));
    thOut[v] = (*H - CM) / ABM;
    i++;
  }

  /* handle internal points */

  for (; i < (isEven ? v-1 : v); i++) {
    j = isEven ? i : i-1; /* points at j and j+1 */
    fcj = isEven ? 2*i : 2*i - 1; /* fc points at fcj and fcj+1 */

    if (isLeak && ileak==i) {
      if (isPos) {
	pp = *pleak;
	pm = 0.0;
      }	else {
	pp = 0.0;
	pm = *pleak;
      }
    } else pp = pm = 0.0;

    CP = thIn[j] + cc2 * (1.0-pp) * thIn[v+j];
    ABP = cc2 + fc[fcj] * cc1 * fabs((1.0-pp) * thIn[v+j]);

    CM = thIn[j+1] - cc2 * thIn[v+j+1] / (1.0-pm);
    ABM = cc2 + fc[fcj+1] * cc1 * fabs(thIn[v+j+1] / (1.0-pm));

    thOut[i] = (CP * ABM + CM * ABP) / (ABP + ABM);
    thOut[v+i] = (CP - CM) / (ABP + ABM);
  }

  /* handle downstream boundary if even time-point */

  if (isEven) {

    if (isLeak && ileak==v-1)
      pp = *pleak;
    else pp = 0.0;

    CP = thIn[v-1] + cc2 * (1.0-pp) * thIn[*n];
    ABP = cc2 + fc[*n-1] * cc1 * fabs((1.0-pp) * thIn[*n]);
    if (*K < 0.0) {
      /* valve closed */
      thOut[*n] = 0.0;
      thOut[v-1] = CP;
    } else if (*K == 0.0) {
      /* valve wide open */
      thOut[*n] = CP / ABP;
      thOut[v-1] = 0.0;
    } else {
      /* valve partly open */
      double aa, q;
      aa = *K / (2.0 * *G);
      q = (ABP + sqrt(ABP*ABP + 4.0 * aa * CP))/2.0;
      thOut[*n] = CP / q;
      thOut[v-1] = aa * thIn[*n] * thIn[*n];
    }
  }
}
